@@ -1,31 +1,35 @@
-import os
-import sys
 import errno
-import logging
-import socket
-import select
-import threading
+import json
 import logging
 import logging.config
-import json
+import os
+import os.path
+import pkgutil
+import select
+import signal
+import socket
+import sys
+import threading
+import time
+import six
+import voltron
+from flask import Flask, Response, make_response, redirect, render_template, request
+from werkzeug.serving import BaseWSGIServer, ThreadedWSGIServer, WSGIRequestHandler
+from werkzeug.middleware.dispatcher import DispatcherMiddleware
+from werkzeug.middleware.shared_data import SharedDataMiddleware
+from requests import ConnectionError
+
+
+# import pysigset
+
+from .api import *
+from .plugin import *
+
 try:
     import requests_unixsocket as requests
 except:
     import requests
-import threading
-import os.path
-import pkgutil
 
-from werkzeug.serving import WSGIRequestHandler, BaseWSGIServer, ThreadedWSGIServer
-from werkzeug.wsgi import SharedDataMiddleware, DispatcherMiddleware
-
-from flask import Flask, request, Response, render_template, make_response, redirect
-
-import voltron
-from .api import *
-from .plugin import *
-
-import six
 if six.PY2:
     if sys.platform == 'win32':
         from SocketServer import ThreadingMixIn
@@ -39,11 +43,15 @@ else:
         from six.moves.socketserver import UnixStreamServer, ThreadingMixIn
     from six.moves.BaseHTTPServer import HTTPServer
 
+ThreadingMixIn.daemon_threads = True
+
 try:
     from voltron_web import app as ui_app
 except:
     ui_app = None
 
+logging.getLogger("requests").setLevel(logging.WARNING)
+
 
 def get_loader(name):
     try:
@@ -142,9 +150,11 @@ class Server(object):
         )
 
         def run_listener(name, cls, arg):
+            # with pysigset.suspended_signals(signal.SIGCHLD):
             log.debug("Starting listener for {} socket on {}".format(name, str(arg)))
             s = cls(*arg)
             t = threading.Thread(target=s.serve_forever)
+            t.daemon = True
             t.start()
             self.threads.append(t)
             self.listeners.append(s)
@@ -167,7 +177,9 @@ class Server(object):
         Stop the server.
         """
         log.debug("Stopping listeners")
+        self.queue_lock.acquire()
         for s in self.listeners:
+            log.debug("Stopping {}".format(s))
             s.shutdown()
             s.socket.close()
         self.cancel_queue()
@@ -176,60 +188,64 @@ class Server(object):
         self.listeners = []
         self.threads = []
         self.is_running = False
+        self.queue_lock.release()
         log.debug("Listeners stopped and threads joined")
 
     def handle_request(self, data):
         req = None
         res = None
 
-        # make sure we have a debugger, or we're gonna have a bad time
-        if voltron.debugger:
-            # parse incoming request with the top level APIRequest class so we can determine the request type
-            try:
-                req = APIRequest(data=data)
-            except Exception as e:
-                req = None
-                log.exception("Exception raised while parsing API request: {} {}".format(type(e), e))
-
-            if req:
-                # instantiate the request class
+        if self.is_running:
+            # make sure we have a debugger, or we're gonna have a bad time
+            if voltron.debugger:
+                # parse incoming request with the top level APIRequest class so we can determine the request type
                 try:
-                    log.debug("data = {}".format(data))
-                    req = api_request(req.request, data=data)
+                    req = APIRequest(data=data)
                 except Exception as e:
-                    log.exception("Exception raised while creating API request: {} {}".format(type(e), e))
                     req = None
-                if not req:
-                    res = APIPluginNotFoundErrorResponse()
-            else:
-                res = APIInvalidRequestErrorResponse()
-        else:
-            res = APIDebuggerNotPresentErrorResponse()
+                    log.exception("Exception raised while parsing API request: {} {}".format(type(e), e))
 
-        if not res:
-            # no errors so far, queue the request and wait
-            if req and req.block:
-                self.queue_lock.acquire()
-                self.queue.append(req)
-                self.queue_lock.release()
-
-                # When this returns the request will have been processed by the dispatch_queue method on the main
-                # thread (or timed out). We have to do it this way because GDB sucks.
-                req.wait()
-
-                if req.timed_out:
-                    res = APITimedOutErrorResponse()
+                if req:
+                    # instantiate the request class
+                    try:
+                        log.debug("data = {}".format(data))
+                        req = api_request(req.request, data=data)
+                    except Exception as e:
+                        log.exception("Exception raised while creating API request: {} {}".format(type(e), e))
+                        req = None
+                    if not req:
+                        res = APIPluginNotFoundErrorResponse()
                 else:
-                    res = req.response
-
-                # Remove the request from the queue
-                self.queue_lock.acquire()
-                if req in self.queue:
-                    self.queue.remove(req)
-                self.queue_lock.release()
+                    res = APIInvalidRequestErrorResponse()
             else:
-                # non-blocking, dispatch request straight away
-                res = self.dispatch_request(req)
+                res = APIDebuggerNotPresentErrorResponse()
+
+            if not res:
+                # no errors so far, queue the request and wait
+                if req and req.block:
+                    self.queue_lock.acquire()
+                    self.queue.append(req)
+                    self.queue_lock.release()
+
+                    # When this returns the request will have been processed by the dispatch_queue method on the main
+                    # thread (or timed out). We have to do it this way because GDB sucks.
+                    req.wait()
+
+                    if req.timed_out:
+                        res = APITimedOutErrorResponse()
+                    else:
+                        res = req.response
+
+                    # Remove the request from the queue
+                    self.queue_lock.acquire()
+                    if req in self.queue:
+                        self.queue.remove(req)
+                    self.queue_lock.release()
+                else:
+                    # non-blocking, dispatch request straight away
+                    res = self.dispatch_request(req)
+        else:
+            res = APIServerNotRunningErrorResponse()
 
         return res
 
@@ -237,13 +253,11 @@ class Server(object):
         """
         Cancel all requests in the queue so we can exit.
         """
-        self.queue_lock.acquire()
         q = list(self.queue)
         self.queue = []
-        self.queue_lock.release()
         log.debug("Canceling requests: {}".format(q))
         for req in q:
-            req.response = APIServerExitedErrorResponse()
+            req.response = APIServerNotRunningErrorResponse()
         for req in q:
             req.signal()
 
@@ -296,12 +310,25 @@ class VoltronWSGIServer(BaseWSGIServer):
 
     This just needs to exist so we can swallow errors when clients disconnect.
     """
+    clients = []
+
     def finish_request(self, *args):
+        self.clients.append(args[0])
+        log.debug("finish_request({})".format(args))
         try:
             super(VoltronWSGIServer, self).finish_request(*args)
         except socket.error as e:
             log.error("Error in finish_request: {}".format(e))
 
+    def shutdown(self):
+        super(VoltronWSGIServer, self).shutdown()
+        for c in self.clients:
+            try:
+                c.shutdown(socket.SHUT_RD)
+                c.close()
+            except:
+                pass
+
 
 class ThreadedVoltronWSGIServer(ThreadingMixIn, VoltronWSGIServer):
     """
@@ -365,7 +392,8 @@ class Client(object):
     """
     Used by a client (ie. a view) to communicate with the server.
     """
-    def __init__(self, host='127.0.0.1', port=5555, sockfile=None, url=None):
+    def __init__(self, host='127.0.0.1', port=5555, sockfile=None, url=None,
+                 build_requests=None, callback=None, supports_blocking=True):
         """
         Initialise a new client
         """
@@ -374,9 +402,17 @@ class Client(object):
             self.url = url
         elif sockfile:
             self.url = 'http+unix://{}/api/request'.format(sockfile.replace('/', '%2F'))
+        elif voltron.config.view.api_url:
+            self.url = voltron.config.view.api_url
         else:
             self.url = 'http://{}:{}/api/request'.format(host, port)
         self.url = self.url.replace('~', os.path.expanduser('~').replace('/', '%2f'))
+        self.callback = callback
+        self.build_requests = build_requests
+        self.done = False
+        self.server_version = None
+        self.block = False
+        self.supports_blocking = supports_blocking
 
     def send_request(self, request):
         """
@@ -467,3 +503,85 @@ class Client(object):
         res = self.send_request(req)
 
         return res
+
+    def update(self):
+        """
+        Update the display
+        """
+        # build requests for this iteration
+        reqs = self.build_requests()
+        for r in reqs:
+            r.block = self.block
+        results = self.send_requests(*reqs)
+
+        # call callback with the results
+        self.callback(results)
+
+    def run(self, build_requests=None, callback=None):
+        """
+        Run the client in a loop, calling the callback each time the debugger
+        stops.
+        """
+        if callback:
+            self.callback = callback
+        if build_requests:
+            self.build_requests = build_requests
+
+        def normalise_requests_err(e):
+            try:
+                msg = e.message.args[1].strerror
+            except:
+                try:
+                    msg = e.message.args[0]
+                except:
+                    msg = str(e)
+            return msg
+
+        while not self.done:
+            try:
+
+                # get the server version info
+                if not self.server_version:
+                    self.server_version = self.perform_request('version')
+
+                    # if the server supports async mode, use it, as some views may only work in async mode
+                    if self.server_version.capabilities and 'async' in self.server_version.capabilities:
+                        self.update()
+                        self.block = False
+                    elif self.supports_blocking:
+                        self.block = True
+                    else:
+                        raise BlockingNotSupportedError("Debugger requires blocking mode")
+
+                if self.block:
+                    # synchronous requests
+                    self.update()
+                else:
+                    # async requests, block using a null request until the debugger stops again
+                    res = self.perform_request('null', block=True)
+                    if res.is_success:
+                        self.server_version = res
+                        self.update()
+            except ConnectionError as e:
+                self.callback(error='Error: {}'.format(normalise_requests_err(e)))
+                self.server_version = None
+                time.sleep(1)
+
+    def start(self, build_requests=None, callback=None):
+        """
+        Run the client using a background thread.
+        """
+        if callback:
+            self.callback = callback
+        if build_requests:
+            self.build_requests = build_requests
+
+        # spin off requester thread
+        self.sw = threading.Thread(target=self.run)
+        self.sw.start()
+
+    def stop(self):
+        """
+        Stop the background thread.
+        """
+        self.done = True
